@@ -1,8 +1,9 @@
 import math
+from math import pi
+from typing import List
 
 import numexpr
 import numpy
-from numpy import pi
 
 
 def icc(theta: float, a: float, b: float, c: float = 0, d: float = 1) -> float:
@@ -43,32 +44,36 @@ def _split_params(items: numpy.ndarray):
 
 def detect_model(items: numpy.ndarray) -> int:
     """Detects which logistic model an item matrix fits into.
+
     :param items: an item matrix
     :return: an int between 1 and 4 denoting the logistic model of the given item matrix
     """
     a, b, c, d = _split_params(items)
 
-    if any(d != 1): return 4
-    if any(c != 0): return 3
-    if len(set(a)) == 1: return 2
+    if any(d != 1):
+        return 4
+    if any(c != 0):
+        return 3
+    if len(set(a)) > 1:
+        return 2
     return 1
 
 
 def icc_hpc(theta: float, items: numpy.ndarray) -> numpy.ndarray:
     """Implementation of :py:func:`icc` using :py:mod:`numpy` and :py:mod:`numexpr` in which the characteristic
-    function for all items in a `numpy.ndarray` are calculated at once.
+    function for all items in a `numpy.ndarray` are computed at once.
 
     :param theta: the individual's proficiency value.
     :param items: array containing the four item parameters.
     :returns: an array of all item characteristic functions, given the current ``theta``"""
     a, b, c, d = _split_params(items)
 
-    return numexpr.evaluate('c + ((d - c) / (1 + exp((-a * (theta - b)))))')
+    return numexpr.evaluate("c + ((d - c) / (1 + exp((-a * (theta - b)))))")
 
 
 def inf_hpc(theta: float, items: numpy.ndarray):
     """Implementation of :py:func:`inf` using :py:mod:`numpy` and :py:mod:`numexpr` in which the information
-    value for all items in a `numpy.ndarray` are calculated at once.
+    values for all items in a `numpy.ndarray` are computed at once.
 
     :param theta: the individual's proficiency value.
     :param items: array containing the four item parameters.
@@ -76,7 +81,7 @@ def inf_hpc(theta: float, items: numpy.ndarray):
     a, b, c, d = _split_params(items)
     p = icc_hpc(theta, items)
 
-    return numexpr.evaluate('(a ** 2 * (p - c) ** 2 * (d - p) ** 2) / ((d - c) ** 2 * p * (1 - p))')
+    return numexpr.evaluate("(a ** 2 * (p - c) ** 2 * (d - p) ** 2) / ((d - c) ** 2 * p * (1 - p))")
 
 
 def inf(theta: float, a: float, b: float, c: float = 0, d: float = 1) -> float:
@@ -123,7 +128,7 @@ def test_info(theta: float, items: numpy.ndarray) -> float:
     :param items: a matrix containing item parameters.
     :returns: the test information at `theta` for a test represented by `items`.
     """
-    return numpy.sum(inf_hpc(theta, items))
+    return float(numpy.sum(inf_hpc(theta, items)))
 
 
 def var(theta: float, items: numpy.ndarray) -> float:
@@ -141,7 +146,7 @@ def var(theta: float, items: numpy.ndarray) -> float:
     try:
         return 1 / test_info(theta, items)
     except ZeroDivisionError:
-        return float('-inf')
+        return float("-inf")
 
 
 def see(theta: float, items: numpy.ndarray) -> float:
@@ -159,11 +164,11 @@ def see(theta: float, items: numpy.ndarray) -> float:
     try:
         return math.sqrt(var(theta, items))
     except ValueError:
-        return float('inf')
+        return float("inf")
 
 
 def reliability(theta: float, items: numpy.ndarray):
-    """ Computes test reliability [Thissen00]_, given by:
+    """Computes test reliability [Thissen00]_, given by:
 
     .. math:: Rel = 1 - \\frac{1}{I(\\theta)}
 
@@ -222,9 +227,12 @@ def max_info(a: float = 1, b: float = 0, c: float = 0, d: float = 1) -> float:
     else:
         u = -(3 / 4) + ((c + d - 2 * c * d) / 2)
         v = (c + d - 1) / 4
-        x_star = 2 * math.sqrt(-u / 3) * math.cos(
-            (1 / 3) * math.acos(-(v / 2) * math.sqrt(27 / (-math.pow(u, 3)))) + (4 * math.pi / 3)
-        ) + 0.5
+        x_star = (
+            2 * math.sqrt(-u / 3) * math.cos(
+                (1 / 3) * math.acos(-(v / 2) * math.sqrt(27 / (-math.pow(u, 3)))) +
+                (4 * math.pi / 3)
+            ) + 0.5
+        )
 
         return b + (1 / a) * math.log((x_star - c) / (d - x_star))
 
@@ -240,19 +248,19 @@ def max_info_hpc(items: numpy.ndarray):
     if all(d == 1):
         if all(c == 0):
             return b
-        return numexpr.evaluate('b + (1 / a) * log((1 + sqrt(1 + 8 * c)) / 2)')
+        return numexpr.evaluate("b + (1 / a) * log((1 + sqrt(1 + 8 * c)) / 2)")
     else:
-        u = numexpr.evaluate('-(3 / 4) + ((c + d - 2 * c * d) / 2)')
-        v = numexpr.evaluate('(c + d - 1) / 4')
+        u = numexpr.evaluate("-(3 / 4) + ((c + d - 2 * c * d) / 2)")
+        v = numexpr.evaluate("(c + d - 1) / 4")
         x_star = numexpr.evaluate(
-            '2 * sqrt(-u / 3) * cos((1 / 3) * arccos(-(v / 2) * sqrt(27 / -(u ** 3))) + (4 * pi / 3)) + 0.5'
+            "2 * sqrt(-u / 3) * cos((1 / 3) * arccos(-(v / 2) * sqrt(27 / -(u ** 3))) + (4 * pi / 3)) + 0.5"
         )
 
-        return numexpr.evaluate('b + (1 / a) * log((x_star - c) / (d - x_star))')
+        return numexpr.evaluate("b + (1 / a) * log((x_star - c) / (d - x_star))")
 
 
 def log_likelihood(
-    est_theta: float, response_vector: list, administered_items: numpy.ndarray
+    est_theta: float, response_vector: List[bool], administered_items: numpy.ndarray
 ) -> float:
     """Calculates the log-likelihood of an estimated proficiency, given a
     response vector and the parameters of the answered items [Ayala2009]_.
@@ -261,11 +269,11 @@ def log_likelihood(
 
     .. math:: L(X_{Ij} | \\theta_j, a_I, b_I, c_I, d_I) = \\prod_{i=1} ^ I P_{ij}(\\theta)^{X_{ij}} Q_{ij}(\\theta)^{1-X_{ij}}
 
-    For mathematical reasons, finding the maximum of :math:`L(X_{Ij}` includes using the
-    product rule of derivations. Since :math:`L(X_{Ij}` has :math:`j` parts, it can be quite
-    complicated to do so. Also, for computational reasons, the product of probabilities can
-    quickly tend to 0, so it is common to use the log-likelihood in maximization/minimization
-    problems, transforming the product of probabilities in a sum of probabilities:
+    Finding the maximum of :math:`L(X_{Ij})` includes using the product rule of derivations.
+    Since :math:`L(X_{Ij})` has :math:`j` parts, it can be quite complicated to do so. Also,
+    for computational reasons, the product of probabilities can quickly tend to 0, so it is
+    common to use the log-likelihood in maximization/minimization problems, transforming the
+    product of probabilities in a sum of probabilities:
 
      .. math:: \\log L(X_{Ij} | \\theta_j, a_I, b_I, c_I, d_I) = \\sum_{i=1} ^ I
                \\left\\lbrace x_{ij} \\log P_{ij}(\\theta)+ (1 - x_{ij}) \\log
@@ -278,13 +286,13 @@ def log_likelihood(
     """
     if len(response_vector) != administered_items.shape[0]:
         raise ValueError(
-            'Response vector and administered items must have the same number of items'
+            "Response vector and administered items must have the same number of items"
         )
     if len(set(response_vector) - {True, False}) > 0:
-        raise ValueError('Response vector must contain only Boolean elements')
+        raise ValueError("Response vector must contain only Boolean elements")
 
     ps = icc_hpc(est_theta, administered_items)
-    ll = numexpr.evaluate('sum(where(response_vector, log(ps), log(1 - ps)))')
+    ll = numexpr.evaluate("sum(where(response_vector, log(ps), log(1 - ps)))")
 
     return ll
 
@@ -293,9 +301,7 @@ def negative_log_likelihood(est_theta: float, *args) -> float:
     """Function used by :py:mod:`scipy.optimize` optimization functions that tend to minimize
     values, instead of maximizing them. Calculates the negative log-likelihood of a proficiency
     value, given a response vector and the parameters of the administered items. The value of
-    :py:func:`negative_log_likelihood` is simply the value of :math:`-` :py:func:`log_likelihood` or, mathematically:
-
-    .. math:: - \\log L(X_{Ij} | \\theta_j, a_I, b_I, c_I, d_I)
+    :py:func:`negative_log_likelihood` is simply the value of :math:`-` :py:func:`log_likelihood`.
 
     :param est_theta: estimated proficiency value
 
@@ -310,32 +316,32 @@ def negative_log_likelihood(est_theta: float, *args) -> float:
 
 def normalize_item_bank(items: numpy.ndarray) -> numpy.ndarray:
     """Normalize an item matrix so that it conforms to the standard used by catsim.
-    The item matrix must have dimension nx3, in which column 1 represents item discrimination,
-    column 2 represents item difficulty, column 3 represents the pseudo-guessing parameter and
-    column 4 represents the item upper asymptote.
+    The item matrix must have dimension :math:`n \\times 4`, in which column 1 represents item
+    discrimination, column 2 represents item difficulty, column 3 represents the pseudo-guessing
+    parameter and column 4 represents the item upper asymptote.
 
     If the matrix has one column, it is assumed to be the difficulty column and the other
-    two columns are added such that items simulate the 1-parameter logistic model.
+    three columns are added such that items simulate the 1-parameter logistic model.
 
     If the matrix has two columns, they are assumed to be the discrimination and difficulty
-    columns, respectively. The pseudo-guessing column is added such that items simulate
-    the 2-parameter logistic model.
+    columns, respectively. The pseudo-guessing and upper asymptote columns are added such that
+    items simulate the 2-parameter logistic model.
 
     If the matrix has three columns, they are assumed to be the discrimination, difficulty
     and pseudo-guessing columns, respectively. The upper asymptote column is added such that
     items simulate the 3-parameter logistic model.
 
     :param items: the item matrix.
-    :returns: an nx4 item matrix conforming to the 4 parameter logistic model.
+    :returns: an :math:`n \\times 4` item matrix conforming to the 4 parameter logistic model.
     """
     if len(items.shape) == 1:
         items = numpy.expand_dims(items, axis=0)
     if items.shape[1] == 1:
-        items = numpy.append(numpy.ones((items.shape[0])), items, axis=1)
+        items = numpy.append(numpy.ones((items.shape[0], 1)), items, axis=1)
     if items.shape[1] == 2:
-        items = numpy.append(items, numpy.zeros((items.shape[0])), axis=1)
+        items = numpy.append(items, numpy.zeros((items.shape[0], 1)), axis=1)
     if items.shape[1] == 3:
-        items = numpy.append(items, numpy.ones((items.shape[0])), axis=1)
+        items = numpy.append(items, numpy.ones((items.shape[0], 1)), axis=1)
 
     return items
 
@@ -346,43 +352,43 @@ def validate_item_bank(items: numpy.ndarray, raise_err: bool = False):
     discrimination, column 2 represents item difficulty, column 3 represents the
     pseudo-guessing parameter and column 4 represents the item upper asymptote.
 
-    The item matrix must have at least one line, exactly four columns and
-    :math:`\\forall i \\in I , a_i > 0 \\wedge 0 < c_i < 1 \\wedge 0 < d_i < 1`
+    The item matrix must have at least one line, representing an item, and exactly four columns, representing the discrimination, difficulty, pseudo-guessing and upper asymptote parameters (:math:`a`, :math:`b`, :math:`c` and :math:`d`), respectively. The item matrix is considered valid if, for all items in the matrix,
+    :math:`a > 0 \\wedge 0 < c < 1 \\wedge 0 < d < 1`.
 
     :param items: the item matrix.
     :param raise_err: whether to raise an error in case the validation fails or
                       just print the error message to standard output.
     """
-    if type(items) is not numpy.ndarray:
-        raise ValueError('Item matrix is not of type {0}'.format(numpy.ndarray))
+    if not isinstance(items, numpy.ndarray):
+        raise ValueError("Item matrix is not of type {0}".format(numpy.ndarray))
 
-    err = ''
+    err = ""
 
     if len(items.shape) == 1:
-        err += 'Item matrix has only one dimension.'
+        err += "Item matrix has only one dimension."
     elif items.shape[1] > 4:
         print(
-            '\nItem matrix has more than 4 columns. catsim tends to add \
-            columns to the matrix during the simulation, so it\'s not a good idea to keep them.'
+            "\nItem matrix has more than 4 columns. catsim tends to add \
+            columns to the matrix during the simulation, so it's not a good idea to keep them."
         )
     elif items.shape[1] < 4:
         if items.shape[1] == 1:
-            err += '\nItem matrix has no discrimination, pseudo-guessing or upper asymptote parameter columns'
+            err += "\nItem matrix has no discrimination, pseudo-guessing or upper asymptote parameter columns"
         elif items.shape[1] == 2:
-            err += '\nItem matrix has no pseudo-guessing or upper asymptote parameter columns'
+            err += "\nItem matrix has no pseudo-guessing or upper asymptote parameter columns"
         elif items.shape[1] == 3:
-            err += '\nItem matrix has no upper asymptote parameter column'
+            err += "\nItem matrix has no upper asymptote parameter column"
     else:
         if any(items[:, 0] < 0):
-            err += '\nThere are items with discrimination < 0'
+            err += "\nThere are items with discrimination < 0"
         if any(items[:, 2] < 0):
-            err += '\nThere are items with pseudo-guessing < 0'
+            err += "\nThere are items with pseudo-guessing < 0"
         if any(items[:, 2] > 1):
-            err += '\nThere are items with pseudo-guessing > 1'
+            err += "\nThere are items with pseudo-guessing > 1"
         if any(items[:, 3] > 1):
-            err += '\nThere are items with upper asymptote > 1'
+            err += "\nThere are items with upper asymptote > 1"
         if any(items[:, 3] < 0):
-            err += '\nThere are items with upper asymptote < 0'
+            err += "\nThere are items with upper asymptote < 0"
 
     if len(err) > 0 and raise_err:
         raise ValueError(err)
