@@ -1,28 +1,17 @@
 from abc import abstractmethod
+from typing import List, Union
 from warnings import warn
 
 import numpy
 from scipy.integrate import quad
 
 from . import irt
-from .simulation import Selector, FiniteSelector
-
-
-def _nearest(array: list, value) -> numpy.ndarray:
-    """Returns the indexes of values in `array` that are closest to `value`
-    
-    :param array: an array of numeric values
-    :param value: a numerical value
-    :return: an array containing the indexes of numbers in `array`,
-             according to how close their are to `value`
-    """
-    array = numpy.asarray(array)
-    return numpy.abs(array - value).argsort()
+from .simulation import FiniteSelector, Selector
 
 
 class MaxInfoSelector(Selector):
     """Selector that returns the first non-administered item with maximum information, given an estimated theta
-       
+
     :param r_max: maximum exposure rate for items
     """
 
@@ -31,7 +20,7 @@ class MaxInfoSelector(Selector):
         self._r_max = r_max
 
     def __str__(self):
-        return 'Maximum Information Selector'
+        return "Maximum Information Selector"
 
     @property
     def r_max(self):
@@ -41,10 +30,10 @@ class MaxInfoSelector(Selector):
         self,
         index: int = None,
         items: numpy.ndarray = None,
-        administered_items: list = None,
+        administered_items: List[int] = None,
         est_theta: float = None,
         **kwargs
-    ) -> int:
+    ) -> Union[int, None]:
         """Returns the index of the next item to be administered.
 
         :param index: the index of the current examinee in the simulator.
@@ -54,37 +43,34 @@ class MaxInfoSelector(Selector):
         :param est_theta: a float containing the current estimated proficiency
         :returns: index of the next item to be applied or `None` if there are no more items in the item bank.
         """
-        if (index is None or self.simulator is None
-            ) and (items is None or administered_items is None or est_theta is None):
-            raise ValueError(
-                'Either pass an index for the simulator or all of the other optional parameters to use this component independently.'
-            )
+        items, administered_items, est_theta = self._prepare_args(
+            return_items=True,
+            return_est_theta=True,
+            index=index,
+            items=items,
+            administered_items=administered_items,
+            est_theta=est_theta,
+            **kwargs
+        )
 
-        if items is None and administered_items is None and est_theta is None:
-            items = self.simulator.items
-            administered_items = self.simulator.administered_items[index]
-            est_theta = self.simulator.latest_estimations[index]
+        assert items is not None
+        assert administered_items is not None
+        assert est_theta is not None
 
-        # first, we'll order items by their information value
-        if irt.detect_model(items) <= 2:
-            # when the logistic model has the number of parameters <= 2,
-            # all items have highest information where theta = b
-            ordered_items = _nearest(items[:, 1], est_theta)
-        else:
-            # else, we'll have to calculate the theta value where information is maximum
-            inf_values = irt.max_info_hpc(items)
-            ordered_items = _nearest(inf_values, est_theta)
+        # sort items by their information value
+        ordered_items = self._sort_by_info(items, est_theta)
+        # remove administered ones
+        valid_indexes = self._get_non_administered(ordered_items, administered_items)
 
-        valid_indexes = [x for x in ordered_items if x not in administered_items]
         if len(valid_indexes) == 0:
-            warn('There are no more items to be applied.')
+            warn("There are no more items to be applied.")
             return None
 
         # gets the indexes and information values from the items with r < rmax
         valid_indexes_low_r = valid_indexes
         if items.shape[1] < 5:
             warn(
-                'This selector needs an item matrix with at least 5 columns, with the last one representing item exposure rate. Since this column is absent, it will presume all items have exposure rates = 0'
+                "This selector needs an item matrix with at least 5 columns, with the last one representing item exposure rate. Since this column is absent, it will presume all items have exposure rates = 0"
             )
         else:
             valid_indexes_low_r = [
@@ -107,16 +93,16 @@ class UrrySelector(Selector):
         super().__init__()
 
     def __str__(self):
-        return 'Urry Selector'
+        return "Urry Selector"
 
     def select(
         self,
         index: int = None,
         items: numpy.ndarray = None,
-        administered_items: list = None,
+        administered_items: List[int] = None,
         est_theta: float = None,
         **kwargs
-    ) -> int:
+    ) -> Union[int, None]:
         """Returns the index of the next item to be administered.
 
         :param index: the index of the current examinee in the simulator.
@@ -126,22 +112,25 @@ class UrrySelector(Selector):
         :param est_theta: a float containing the current estimated proficiency
         :returns: index of the next item to be applied or `None` if there are no more items in the item bank.
         """
-        if (index is None or self.simulator is None
-            ) and (items is None or administered_items is None or est_theta is None):
-            raise ValueError(
-                'Either pass an index for the simulator or all of the other optional parameters to use this component independently.'
-            )
+        items, administered_items, est_theta = self._prepare_args(
+            return_items=True,
+            return_est_theta=True,
+            index=index,
+            items=items,
+            administered_items=administered_items,
+            est_theta=est_theta,
+            **kwargs
+        )
 
-        if items is None and administered_items is None and est_theta is None:
-            items = self.simulator.items
-            administered_items = self.simulator.administered_items[index]
-            est_theta = self.simulator.latest_estimations[index]
+        assert est_theta is not None
+        assert administered_items is not None
+        assert items is not None
 
-        ordered_items = _nearest(items[:, 1], est_theta)
-        valid_indexes = [x for x in ordered_items if x not in administered_items]
+        ordered_items = self._sort_by_b(items, est_theta)
+        valid_indexes = self._get_non_administered(ordered_items, administered_items)
 
         if len(valid_indexes) == 0:
-            warn('There are no more items to be applied.')
+            warn("There are no more items to be applied.")
             return None
 
         return valid_indexes[0]
@@ -154,9 +143,9 @@ class LinearSelector(FiniteSelector):
     :param indexes: the indexes of the items that will be returned in order"""
 
     def __str__(self):
-        return 'Linear Selector'
+        return "Linear Selector"
 
-    def __init__(self, indexes: list):
+    def __init__(self, indexes: List[int]):
         super().__init__(len(indexes))
         self._indexes = indexes
         self._current = 0
@@ -169,34 +158,35 @@ class LinearSelector(FiniteSelector):
     def current(self):
         return self._current
 
-    def select(self, index: int = None, administered_items: list = None, **kwargs) -> int:
+    def select(self,
+               index: int = None,
+               administered_items: List[int] = None,
+               **kwargs) -> Union[int, None]:
         """Returns the index of the next item to be administered.
 
         :param index: the index of the current examinee in the simulator.
         :param administered_items: a list containing the indexes of items that were already administered
         :returns: index of the next item to be applied or `None` if there are no more items in the item bank.
         """
-        if (index is None or self.simulator is None) and (administered_items is None):
-            raise ValueError(
-                'Either pass an index for the simulator or all of the other optional parameters to use this component independently.'
-            )
+        (administered_items,
+         ) = self._prepare_args(index=index, administered_items=administered_items, **kwargs)
 
-        if administered_items is None:
-            administered_items = self.simulator.administered_items[index]
+        valid_indexes = self._get_non_administered(self._indexes, administered_items)
 
-        if set(self._indexes) <= set(administered_items):
+        if len(valid_indexes) == 0:
             warn(
-                'A new index was asked for, but there are no more item indexes to present.\nCurrent item:\t\t\t{0}\nItems to be administered:\t{1} (size: {2})\nAdministered items:\t\t{3} (size: {4})'
+                "A new index was asked for, but there are no more item indexes to present.\nCurrent item:\t\t\t{0}\nItems to be administered:\t{1} (size: {2})\nAdministered items:\t\t{3} (size: {4})"
                 .format(
-                    self._current, sorted(self._indexes), len(self._indexes),
-                    sorted(administered_items), len(administered_items)
+                    self._current,
+                    sorted(self._indexes),
+                    len(self._indexes),
+                    sorted(administered_items),
+                    len(administered_items),
                 )
             )
             return None
 
-        selected_item = [x for x in self._indexes if x not in administered_items][0]
-
-        return selected_item
+        return valid_indexes[0]
 
 
 class RandomSelector(Selector):
@@ -205,7 +195,7 @@ class RandomSelector(Selector):
     :param replace: whether to select an item that has already been selected before for this examinee."""
 
     def __str__(self):
-        return 'Random Selector'
+        return "Random Selector"
 
     def __init__(self, replace: bool = False):
         super().__init__()
@@ -215,9 +205,9 @@ class RandomSelector(Selector):
         self,
         index: int = None,
         items: numpy.ndarray = None,
-        administered_items: list = None,
+        administered_items: List[int] = None,
         **kwargs
-    ) -> int:
+    ) -> Union[int, None]:
         """Returns the index of the next item to be administered.
 
         :param index: the index of the current examinee in the simulator.
@@ -226,19 +216,20 @@ class RandomSelector(Selector):
         :param administered_items: a list containing the indexes of items that were already administered
         :returns: index of the next item to be applied or `None` if there are no more items in the item bank.
         """
-        if (index is None or
-            self.simulator is None) and (items is None or administered_items is None):
-            raise ValueError(
-                'Either pass an index for the simulator or all of the other optional parameters to use this component independently.'
-            )
+        items, administered_items = self._prepare_args(
+            return_items=True,
+            index=index,
+            items=items,
+            administered_items=administered_items,
+            **kwargs
+        )
 
-        if items is None and administered_items is None:
-            items = self.simulator.items
-            administered_items = self.simulator.administered_items[index]
+        assert items is not None
+        assert administered_items is not None
 
         if len(administered_items) >= items.shape[0] and not self._replace:
             warn(
-                'A new item was asked for, but there are no more items to present.\nAdministered items:\t{0}\nItem bank size:\t{1}'
+                "A new item was asked for, but there are no more items to present.\nAdministered items:\t{0}\nItem bank size:\t{1}"
                 .format(len(administered_items), items.shape[0])
             )
             return None
@@ -246,7 +237,10 @@ class RandomSelector(Selector):
         if self._replace:
             return numpy.random.choice(items.shape[0])
         else:
-            return numpy.random.choice(list(set(range(items.shape[0])) - set(administered_items)))
+            valid_indexes = self._get_non_administered(
+                list(range(items.shape[0])), administered_items
+            )
+            return numpy.random.choice(valid_indexes)
 
 
 class ClusterSelector(Selector):
@@ -279,7 +273,7 @@ class ClusterSelector(Selector):
     """
 
     def __str__(self):
-        return 'Cluster Selector'
+        return "Cluster Selector"
 
     @property
     def r_max(self):
@@ -299,23 +293,23 @@ class ClusterSelector(Selector):
 
     def __init__(
         self,
-        clusters: list,
-        method: str = 'item_info',
+        clusters: List[int],
+        method: str = "item_info",
         r_max: float = 1,
-        r_control: str = 'passive'
+        r_control: str = "passive",
     ):
         super().__init__()
-        available_methods = ['item_info', 'cluster_info', 'weighted_info']
+        available_methods = ["item_info", "cluster_info", "weighted_info"]
         if method not in available_methods:
             raise ValueError(
-                '{0} is not a valid cluster selection method; choose one from {1}'.format(
+                "{0} is not a valid cluster selection method; choose one from {1}".format(
                     method, available_methods
                 )
             )
-        available_rcontrol = ['passive', 'aggressive']
+        available_rcontrol = ["passive", "aggressive"]
         if r_control not in available_rcontrol:
             raise ValueError(
-                '{0} is not a valid item exposure control method; choose one from {1}'.format(
+                "{0} is not a valid item exposure control method; choose one from {1}".format(
                     r_control, available_rcontrol
                 )
             )
@@ -329,10 +323,10 @@ class ClusterSelector(Selector):
         self,
         index: int = None,
         items: numpy.ndarray = None,
-        administered_items: list = None,
+        administered_items: List[int] = None,
         est_theta: float = None,
         **kwargs
-    ) -> int:
+    ) -> Union[int, None]:
         """Returns the index of the next item to be administered.
 
         :param index: the index of the current examinee in the simulator.
@@ -342,25 +336,28 @@ class ClusterSelector(Selector):
         :param est_theta: a float containing the current estimated proficiency
         :returns: index of the next item to be applied.
         """
-        if (index is None or self.simulator is None
-            ) and (items is None or administered_items is None or est_theta is None):
-            raise ValueError(
-                'Either pass an index for the simulator or all of the other optional parameters to use this component independently.'
-            )
+        items, administered_items, est_theta = self._prepare_args(
+            return_items=True,
+            return_est_theta=True,
+            index=index,
+            items=items,
+            administered_items=administered_items,
+            est_theta=est_theta,
+            **kwargs
+        )
 
-        if items is None and administered_items is None and est_theta is None:
-            items = self.simulator.items
-            administered_items = self.simulator.administered_items[index]
-            est_theta = self.simulator.latest_estimations[index]
+        assert items is not None
+        assert administered_items is not None
+        assert est_theta is not None
 
         selected_cluster = None
         existent_clusters = set(self._clusters)
 
         # this part of the code selects the cluster from which the item at
         # the current point of the test will be chosen
-        if self._method == 'item_info':
+        if self._method == "item_info":
             # get the item indexes sorted by their information value
-            infos = _nearest(irt.max_info_hpc(items), est_theta)
+            infos = self._sort_by_info(items, est_theta)
 
             evaluated_clusters = set()
 
@@ -394,10 +391,10 @@ class ClusterSelector(Selector):
                 selected_cluster = self._clusters[max_info_item]
                 break
 
-        elif self._method in ['cluster_info', 'weighted_info']:
+        elif self._method in ["cluster_info", "weighted_info"]:
             # calculates the cluster information, depending on the method
             # selected
-            if self._method == 'cluster_info':
+            if self._method == "cluster_info":
                 cluster_infos = ClusterSelector.sum_cluster_infos(est_theta, items, self._clusters)
             else:
                 cluster_infos = ClusterSelector.weighted_cluster_infos(
@@ -412,10 +409,10 @@ class ClusterSelector(Selector):
                     cluster for (inf_value, cluster) in sorted(
                         zip(cluster_infos, set(self._clusters)),
                         key=lambda pair: pair[0],
-                        reverse=True
+                        reverse=True,
                     )
                 ],
-                dtype=float
+                dtype=float,
             )
 
             # walks through the sorted clusters in order
@@ -441,19 +438,17 @@ class ClusterSelector(Selector):
 
         # gets the indexes and information values from the items in the
         # selected cluster that have not been administered
-        valid_indexes = [
-            index
-            for index in numpy.nonzero([cluster == selected_cluster
-                                        for cluster in self._clusters])[0]
-            if index not in administered_items
-        ]
+        valid_indexes = self._get_non_administered(
+            numpy.nonzero([cluster == selected_cluster for cluster in self._clusters])[0],
+            administered_items,
+        )
 
         # gets the indexes and information values from the items in the
         # selected cluster with r < rmax that have not been administered
         valid_indexes_low_r = valid_indexes
         if items.shape[1] < 5:
             warn(
-                'This selector needs an item matrix with at least 5 columns, with the last one representing item exposure rate. Since this column is absent, it will presume all items have exposure rates = 0'
+                "This selector needs an item matrix with at least 5 columns, with the last one representing item exposure rate. Since this column is absent, it will presume all items have exposure rates = 0"
             )
         else:
             valid_indexes_low_r = [
@@ -469,7 +464,7 @@ class ClusterSelector(Selector):
         # if all items in the selected cluster have exceed their r values,
         # select the one with smallest r, regardless of information
         else:
-            if self._r_control == 'passive':
+            if self._r_control == "passive":
                 inf_values = irt.inf_hpc(est_theta, items[valid_indexes])
                 selected_item = valid_indexes[numpy.nonzero(inf_values == max(inf_values))[0][0]]
             else:
@@ -478,7 +473,7 @@ class ClusterSelector(Selector):
         return selected_item
 
     @staticmethod
-    def sum_cluster_infos(theta: float, items: numpy.ndarray, clusters: list) -> numpy.ndarray:
+    def sum_cluster_infos(theta: float, items: numpy.ndarray, clusters: List[int]) -> numpy.ndarray:
         """Returns the sum of item information values, separated by cluster
 
         :param theta: an examinee's :math:`\\theta` value
@@ -497,7 +492,9 @@ class ClusterSelector(Selector):
         return cluster_infos
 
     @staticmethod
-    def weighted_cluster_infos(theta: float, items: numpy.ndarray, clusters: list) -> numpy.ndarray:
+    def weighted_cluster_infos(
+        theta: float, items: numpy.ndarray, clusters: List[int]
+    ) -> numpy.ndarray:
         """Returns the weighted sum of item information values, separated by cluster.
         The weight is the number of items in each cluster.
 
@@ -516,7 +513,7 @@ class ClusterSelector(Selector):
         return cluster_infos
 
     @staticmethod
-    def sum_cluster_params(items: numpy.ndarray, c: list):
+    def sum_cluster_params(items: numpy.ndarray, c: List[int]):
         """Returns the sum of item parameter values for each cluster
 
         :param items: a matrix containing item parameters in the format that `catsim` understands
@@ -537,7 +534,7 @@ class ClusterSelector(Selector):
         return averages
 
     @staticmethod
-    def avg_cluster_params(items: numpy.ndarray, c: list):
+    def avg_cluster_params(items: numpy.ndarray, c: List[int]):
         """Returns the average values of item parameters by cluster
 
         :param items:
@@ -560,7 +557,7 @@ class ClusterSelector(Selector):
 class StratifiedSelector(FiniteSelector):
 
     def __str__(self):
-        return 'General Stratified Selector'
+        return "General Stratified Selector"
 
     def __init__(self, test_size):
         super().__init__(test_size)
@@ -579,9 +576,9 @@ class StratifiedSelector(FiniteSelector):
         self,
         index: int = None,
         items: numpy.ndarray = None,
-        administered_items: list = None,
+        administered_items: List[int] = None,
         **kwargs
-    ) -> int:
+    ) -> Union[int, None]:
         """Returns the index of the next item to be administered.
 
         :param index: the index of the current examinee in the simulator.
@@ -589,24 +586,26 @@ class StratifiedSelector(FiniteSelector):
         :param administered_items: a list containing the indexes of items that were already administered
         :returns: index of the next item to be applied or `None` if there are no more strata to get items from.
         """
-        if (index is None or
-            self.simulator is None) and (items is None or administered_items is None):
-            raise ValueError(
-                'Either pass an index for the simulator or all of the other optional parameters to use this component independently.'
-            )
+        items, administered_items = self._prepare_args(
+            return_items=True,
+            index=index,
+            items=items,
+            administered_items=administered_items,
+            **kwargs
+        )
 
-        if items is None and administered_items is None:
-            items = self.simulator.items
-            administered_items = self.simulator.administered_items[index]
+        assert items is not None
+        assert administered_items is not None
 
         # select the item in the correct layer, according to the point in the test the examinee is
-        slices = numpy.linspace(0, items.shape[0], self._test_size, endpoint=False, dtype='i')
+        slices = numpy.linspace(0, items.shape[0], self._test_size, endpoint=False, dtype="i")
 
         try:
             pointer = slices[len(administered_items)]
-            max_pointer = items.shape[0] if len(
-                administered_items
-            ) == self._test_size - 1 else slices[len(administered_items) + 1]
+            max_pointer = (
+                items.shape[0] if len(administered_items) == self._test_size -
+                1 else slices[len(administered_items) + 1]
+            )
         except IndexError:
             warn(
                 "{0}: test size is larger than was informed to the selector\nLength of administered items:\t{1}\nTotal length of the test:\t{2}\nNumber of slices:\t{3}"
@@ -614,8 +613,8 @@ class StratifiedSelector(FiniteSelector):
             )
             return None
 
-        organized_items = self._organized_items if self._organized_items is not None else self.sort_items(
-            items
+        organized_items = (
+            self._organized_items if self._organized_items is not None else self.sort_items(items)
         )
 
         # if the selected item has already been administered, select the next one
@@ -623,7 +622,7 @@ class StratifiedSelector(FiniteSelector):
             pointer += 1
             if pointer == max_pointer:
                 raise ValueError(
-                    'There are no more items to be selected from stratum {0}'.format(
+                    "There are no more items to be selected from stratum {0}".format(
                         slices[len(administered_items)]
                     )
                 )
@@ -648,7 +647,7 @@ class AStratSelector(StratifiedSelector):
     """
 
     def __str__(self):
-        return 'a-Stratified Selector'
+        return "a-Stratified Selector"
 
     def __init__(self, test_size):
         super().__init__(test_size)
@@ -678,7 +677,7 @@ class AStratBBlockSelector(StratifiedSelector):
     """
 
     def __str__(self):
-        return 'a-Stratified b-Blocking Selector'
+        return "a-Stratified b-Blocking Selector"
 
     def __init__(self, test_size):
         super().__init__(test_size)
@@ -709,7 +708,7 @@ class MaxInfoStratSelector(StratifiedSelector):
     """
 
     def __str__(self):
-        return 'Maximum Information Stratification Selector'
+        return "Maximum Information Stratification Selector"
 
     def __init__(self, test_size):
         super().__init__(test_size)
@@ -745,7 +744,7 @@ class MaxInfoBBlockSelector(StratifiedSelector):
     """
 
     def __str__(self):
-        return 'Maximum Information Stratification with b-Blocking Selector'
+        return "Maximum Information Stratification with b-Blocking Selector"
 
     def __init__(self, test_size):
         super().__init__(test_size)
@@ -771,7 +770,7 @@ class The54321Selector(FiniteSelector):
                       this parameter to set the bin size"""
 
     def __str__(self):
-        return '5-4-3-2-1 Selector'
+        return "5-4-3-2-1 Selector"
 
     def __init__(self, test_size):
         super().__init__(test_size)
@@ -780,10 +779,10 @@ class The54321Selector(FiniteSelector):
         self,
         index: int = None,
         items: numpy.ndarray = None,
-        administered_items: list = None,
+        administered_items: List[int] = None,
         est_theta: float = None,
         **kwargs
-    ) -> int:
+    ) -> Union[int, None]:
         """Returns the index of the next item to be administered.
 
         :param index: the index of the current examinee in the simulator.
@@ -793,28 +792,29 @@ class The54321Selector(FiniteSelector):
         :param est_theta: a float containing the current estimated proficiency
         :returns: index of the next item to be applied or `None` if there are no more items in the item bank.
         """
-        if (index is None or self.simulator is None
-            ) and (items is None or administered_items is None or est_theta is None):
-            raise ValueError(
-                'Either pass an index for the simulator or all of the other optional parameters to use this component independently.'
-            )
+        items, administered_items, est_theta = self._prepare_args(
+            return_items=True,
+            return_est_theta=True,
+            index=index,
+            items=items,
+            administered_items=administered_items,
+            est_theta=est_theta,
+            **kwargs
+        )
 
-        if items is None and administered_items is None and est_theta is None:
-            items = self.simulator.items
-            administered_items = self.simulator.administered_items[index]
-            est_theta = self.simulator.latest_estimations[index]
+        assert est_theta is not None
+        assert administered_items is not None
+        assert items is not None
 
         # sort item indexes by their information value descending and remove indexes of administered items
-        organized_items = [
-            x for x in (-irt.inf_hpc(est_theta, items)).argsort() if x not in administered_items
-        ]
-
-        bin_size = self._test_size - len(administered_items)
+        ordered_items = self._sort_by_info(items, est_theta)
+        organized_items = self._get_non_administered(ordered_items, administered_items)
 
         if len(organized_items) == 0:
-            warn('There are no more items to apply.')
+            warn("There are no more items to apply.")
             return None
 
+        bin_size = self._test_size - len(administered_items)
         return numpy.random.choice(organized_items[0:bin_size])
 
 
@@ -829,7 +829,7 @@ class RandomesqueSelector(Selector):
     """
 
     def __str__(self):
-        return 'Randomesque Selector'
+        return "Randomesque Selector"
 
     def __init__(self, bin_size):
         super().__init__()
@@ -843,10 +843,10 @@ class RandomesqueSelector(Selector):
         self,
         index: int = None,
         items: numpy.ndarray = None,
-        administered_items: list = None,
+        administered_items: List[int] = None,
         est_theta: float = None,
         **kwargs
-    ) -> int:
+    ) -> Union[int, None]:
         """Returns the index of the next item to be administered.
 
         :param index: the index of the current examinee in the simulator.
@@ -856,24 +856,26 @@ class RandomesqueSelector(Selector):
         :param est_theta: a float containing the current estimated proficiency
         :returns: index of the next item to be applied or `None` if there are no more items in the item bank.
         """
-        if (index is None or self.simulator is None
-            ) and (items is None or administered_items is None or est_theta is None):
-            raise ValueError(
-                'Either pass an index for the simulator or all of the other optional parameters to use this component independently.'
-            )
+        items, administered_items, est_theta = self._prepare_args(
+            return_items=True,
+            return_est_theta=True,
+            index=index,
+            items=items,
+            administered_items=administered_items,
+            est_theta=est_theta,
+            **kwargs
+        )
 
-        if items is None and administered_items is None and est_theta is None:
-            items = self.simulator.items
-            administered_items = self.simulator.administered_items[index]
-            est_theta = self.simulator.latest_estimations[index]
+        assert est_theta is not None
+        assert administered_items is not None
+        assert items is not None
 
         # sort item indexes by their information value descending and remove indexes of administered items
-        organized_items = [
-            x for x in (-irt.inf_hpc(est_theta, items)).argsort() if x not in administered_items
-        ]
+        ordered_items = self._sort_by_info(items, est_theta)
+        organized_items = self._get_non_administered(ordered_items, administered_items)
 
         if len(organized_items) == 0:
-            warn('There are no more items to apply.')
+            warn("There are no more items to apply.")
             return None
 
         return numpy.random.choice(list(organized_items)[:self._bin_size])
@@ -884,14 +886,14 @@ class IntervalInfoSelector(Selector):
     the integral of the information function at a predetermined ``interval``
     :math:`\\delta` above and below the current :math:`\\hat\\theta` is chosen.
 
-    .. math:: argmax_{i \\in I} \\int_{\\hat\\theta - \\delta}^{\\hat\\theta - \\delta}I_i(\\hat\\theta)
+    .. math:: argmax_{i \\in I} \\int_{\\hat\\theta - \\delta}^{\\hat\\theta + \\delta}I_i(\\hat\\theta)
 
     :param interval: the interval of the integral. If no interval is passed, the
-                     integral is calculated from :math:`[-\\infty, \\infty]`.
+                     integral is computed from :math:`[-\\infty, \\infty]`.
     """
 
     def __str__(self):
-        return 'Interval Information Selector'
+        return "Interval Information Selector"
 
     def __init__(self, interval: float = None):
         super().__init__()
@@ -905,10 +907,10 @@ class IntervalInfoSelector(Selector):
         self,
         index: int = None,
         items: numpy.ndarray = None,
-        administered_items: list = None,
+        administered_items: List[int] = None,
         est_theta: float = None,
         **kwargs
-    ) -> int:
+    ) -> Union[int, None]:
         """Returns the index of the next item to be administered.
 
         :param index: the index of the current examinee in the simulator.
@@ -918,35 +920,38 @@ class IntervalInfoSelector(Selector):
         :param est_theta: a float containing the current estimated proficiency
         :returns: index of the next item to be applied or `None` if there are no more items in the item bank.
         """
-        if (index is None or self.simulator is None
-            ) and (items is None or administered_items is None or est_theta is None):
-            raise ValueError(
-                'Either pass an index for the simulator or all of the other optional parameters to use this component independently.'
-            )
+        items, administered_items, est_theta = self._prepare_args(
+            return_items=True,
+            return_est_theta=True,
+            index=index,
+            items=items,
+            administered_items=administered_items,
+            est_theta=est_theta,
+            **kwargs
+        )
 
-        if items is None and administered_items is None and est_theta is None:
-            items = self.simulator.items
-            administered_items = self.simulator.administered_items[index]
-            est_theta = self.simulator.latest_estimations[index]
+        assert est_theta is not None
+        assert administered_items is not None
+        assert items is not None
 
-        # sort item indexes by the integral of the information function descending and remove indexes of administered items
-        organized_items = [
-            x for x in (
-                -numpy.array(
-                    [
-                        quad(
-                            irt.inf,
-                            est_theta - self._interval,
-                            est_theta + self._interval,
-                            args=(item[0], item[1], item[2], item[3])
-                        )[0] for item in items
-                    ]
-                )
-            ).argsort() if x not in administered_items
-        ]
+        # compute the integral of the information function around an examinee's proficiency
+        information_integral = numpy.array(
+            [
+                quad(
+                    irt.inf,
+                    est_theta - self._interval,
+                    est_theta + self._interval,
+                    args=(item[0], item[1], item[2], item[3]),
+                )[0] for item in items
+            ]
+        )
+        # sort by that integral in descending order
+        ordered_items = (-information_integral).argsort()
+        # remove administered items
+        organized_items = self._get_non_administered(ordered_items, administered_items)
 
         if len(organized_items) == 0:
-            warn('There are no more items to apply.')
+            warn("There are no more items to apply.")
             return None
 
-        return list(organized_items)[0]
+        return organized_items[0]
