@@ -2,12 +2,12 @@
 application of adaptive tests. Most of this module is based on the work of
 [Bar10]_."""
 
-import datetime
 import time
-import tqdm
 from abc import ABCMeta, abstractmethod
+from typing import List, Optional, Union
 
 import numpy
+import tqdm
 
 from . import cat, irt
 
@@ -21,14 +21,14 @@ class Simulable(metaclass=ABCMeta):
 
     @property
     def simulator(self):
-        if self._simulator is not None and type(self._simulator) is not Simulator:
-            raise ValueError('simulator has to be of type catsim.simulation.Simulator')
+        if self._simulator is not None and not isinstance(self._simulator, Simulator):
+            raise ValueError("simulator has to be of type catsim.simulation.Simulator")
         return self._simulator
 
     @simulator.setter
-    def simulator(self, x: 'Simulator'):
-        if type(x) is not Simulator:
-            raise ValueError('simulator has to be of type catsim.simulation.Simulator')
+    def simulator(self, x: "Simulator"):
+        if not isinstance(x, Simulator):
+            raise ValueError("simulator has to be of type catsim.simulation.Simulator")
         self._simulator = x
         self.preprocess()
 
@@ -39,6 +39,39 @@ class Simulable(metaclass=ABCMeta):
         `Simulable` are re-initialized as necessary."""
         pass
 
+    def _prepare_args(
+        self, return_items=False, return_response_vector=False, return_est_theta=False, **kwargs
+    ):
+        using_simulator_props = (kwargs.get("index") is not None and self.simulator is not None)
+        if not using_simulator_props and (
+            kwargs.get("items") is None or kwargs.get("administered_items") is None or
+            (return_est_theta and kwargs.get("est_theta") is None)
+        ):
+            raise ValueError(
+                "Either pass an index for the simulator or all of the other "
+                "optional parameters to use this component independently."
+            )
+
+        result = []
+        if using_simulator_props:
+            index = kwargs["index"]
+            if return_items:
+                result.append(self.simulator.items)
+            result.append(self.simulator.administered_items[index])
+            if return_response_vector:
+                result.append(self.simulator.response_vectors[index])
+            if return_est_theta:
+                result.append(self.simulator.latest_estimations[index])
+        else:
+            if return_items:
+                result.append(kwargs["items"])
+            result.append(kwargs["administered_items"])
+            if return_response_vector:
+                result.append(kwargs["response_vector"])
+            if return_est_theta:
+                result.append(kwargs["est_theta"])
+        return tuple(result)
+
 
 class Initializer(Simulable, metaclass=ABCMeta):
     """Base class for CAT initializers"""
@@ -62,8 +95,56 @@ class Selector(Simulable, metaclass=ABCMeta):
     def __init__(self):
         super().__init__()
 
+    @staticmethod
+    def _get_non_administered(
+        item_indices: List[int], administered_item_indices: List[int]
+    ) -> list:
+        """Gets a list of items that were not administered from a list of indices
+
+        :param item_indices: a list of integers, corresponding to item indices
+        :type item_indices: List[int]
+        :param administered_item_indices: a list of integers, corresponding to the indices of items that were alredy administered to a given examinee
+        :type administered_item_indices: List[int]
+        :return: a list of items, corresponding to the indices that are in `item_indices` but not in `administered_item_indices`, in the same order they were passed in `item_indices`
+        :rtype: List[int]
+        """
+        return [x for x in item_indices if x not in administered_item_indices]
+
+    @staticmethod
+    def _sort_by_info(items: numpy.ndarray, est_theta: float) -> list:
+        """Sort items by their information value, given a proficiency value
+
+        :param items: an item parameter matrix
+        :type items: numpy.ndarray
+        :param est_theta: an examinee's proficiency
+        :type est_theta: float
+        :return: List[int] containing the indices of items, sorted in descending order by their information values (much like the return of `numpy.argsort`)
+        :rtype: List[int]
+        """
+        if irt.detect_model(items) == 1:
+            # when the logistic model has the number of parameters <= 2,
+            # all items have highest information where theta = b
+            ordered_items = Selector._sort_by_b(items, est_theta)
+        else:
+            # else, sort item indexes by their information value descending and remove indexes of administered items
+            ordered_items = [x for x in (-irt.inf_hpc(est_theta, items)).argsort()]
+        return ordered_items
+
+    @staticmethod
+    def _sort_by_b(items: numpy.ndarray, est_theta: float) -> list:
+        """Sort items by how close their difficulty parameter is in relaiton to an examinee's proficiency
+
+        :param items: an item parameter matrix
+        :type items: numpy.ndarray
+        :param est_theta: an examinee's proficiency
+        :type est_theta: float
+        :return: list containing the indices of items, sorted by how close their difficulty parameter is in relaiton to :param:`est_theta` (much like the return of `numpy.argsort`)
+        :rtype: List[int]
+        """
+        return list(numpy.abs(items[:, 1] - est_theta).argsort())
+
     @abstractmethod
-    def select(self, index: int) -> int:
+    def select(self, index: int = None) -> Union[int, None]:
         """Returns the index of the next item to be administered.
 
         :param index: the index of the current examinee in the simulator.
@@ -92,8 +173,11 @@ class FiniteSelector(Selector, metaclass=ABCMeta):
 class Estimator(Simulable, metaclass=ABCMeta):
     """Base class for proficiency estimators"""
 
-    def __init__(self):
+    def __init__(self, verbose: bool = False):
         super().__init__()
+        self._calls = 0
+        self._evaluations = 0
+        self._verbose = verbose
 
     @abstractmethod
     def estimate(self, index: int) -> float:
@@ -105,6 +189,27 @@ class Estimator(Simulable, metaclass=ABCMeta):
         """
         pass
 
+    @property
+    def calls(self):
+        """How many times the estimator has been called to maximize/minimize the log-likelihood function
+
+        :returns: number of times the estimator has been called to maximize/minimize the log-likelihood function"""
+        return self._calls
+
+    @property
+    def evaluations(self):
+        """Total number of times the estimator has evaluated the log-likelihood function during its existence
+
+        :returns: number of function evaluations"""
+        return self._evaluations
+
+    @property
+    def avg_evaluations(self):
+        """Average number of function evaluations for all tests the estimator has been used
+
+        :returns: average number of function evaluations"""
+        return self._evaluations / self._calls
+
 
 class Stopper(Simulable, metaclass=ABCMeta):
     """Base class for CAT stop criterion"""
@@ -134,11 +239,11 @@ class Simulator:
     def __init__(
         self,
         items: numpy.ndarray,
-        examinees,
+        examinees: Union[int, list, numpy.ndarray],
         initializer: Initializer = None,
         selector: Selector = None,
         estimator: Estimator = None,
-        stopper: Stopper = None
+        stopper: Stopper = None,
     ):
         irt.validate_item_bank(items)
 
@@ -146,13 +251,13 @@ class Simulator:
         if items.shape[1] < 5:
             items = numpy.append(items, numpy.zeros([items.shape[0], 1]), axis=1)
 
-        self._duration = 0
+        self._duration = 0.0
         self._items = items
 
-        self._bias = None
-        self._mse = None
-        self._rmse = None
-        self._overlap_rate = None
+        self._bias = 0.0
+        self._mse = 0.0
+        self._rmse = 0.0
+        self._overlap_rate = 0.0
 
         self._initializer = initializer
         self._selector = selector
@@ -160,11 +265,15 @@ class Simulator:
         self._stopper = stopper
 
         # `examinees` is passed to its special setter
-        self.examinees = examinees
+        self._examinees = self._to_distribution(examinees)
 
-        self._estimations = [[] for _ in range(len(self.examinees))]
-        self._administered_items = [[] for _ in range(len(self.examinees))]
-        self._response_vectors = [[] for _ in range(len(self.examinees))]
+        self._estimations = [[] for _ in range(self.examinees.shape[0])]  # type: List[List[int]]
+        self._administered_items = [
+            [] for _ in range(self.examinees.shape[0])
+        ]  # type: List[List[int]]
+        self._response_vectors = [
+            [] for _ in range(self.examinees.shape[0])
+        ]  # type: List[List[bool]]
 
     @property
     def items(self) -> numpy.ndarray:
@@ -195,11 +304,6 @@ class Simulator:
         """Final estimated :math:`\\hat\\theta` values for all examinees."""
         return [ests[-1] if len(ests) > 0 else None for ests in self._estimations]
 
-    @property
-    def examinees(self) -> list:
-        """List containing examinees true proficiency values (:math:`\\theta`)."""
-        return self._examinees
-
     @property
     def duration(self) -> float:
         """Duration of the simulation, in seconds."""
@@ -211,19 +315,19 @@ class Simulator:
         return self._overlap_rate
 
     @property
-    def initializer(self) -> Initializer:
+    def initializer(self) -> Optional[Initializer]:
         return self._initializer
 
     @property
-    def selector(self) -> Selector:
+    def selector(self) -> Optional[Selector]:
         return self._selector
 
     @property
-    def estimator(self) -> Estimator:
+    def estimator(self) -> Optional[Estimator]:
         return self._estimator
 
     @property
-    def stopper(self) -> Stopper:
+    def stopper(self) -> Optional[Stopper]:
         return self._stopper
 
     @property
@@ -249,33 +353,43 @@ class Simulator:
         :py:func:`catsim.cat.rmse`"""
         return self._rmse
 
+    @property
+    def examinees(self) -> numpy.ndarray:
+        """:py:type:numpy.ndarray containing examinees true proficiency values (:math:`\\theta`)."""
+        return self._examinees
+
     @examinees.setter
-    def examinees(self, x):
+    def examinees(self, x: Union[int, list, numpy.ndarray]):
+        self._examinees = self._to_distribution(x)
+
+    def _to_distribution(self, x):
         # generate examinees from a normal distribution
         # if an int was passed
-        if type(x) == int:
+        if isinstance(x, int):
             if self._items is not None:
                 mean = numpy.mean(self._items[:, 1])
                 stddev = numpy.std(self._items[:, 1])
-                self._examinees = numpy.random.normal(mean, stddev, x)
+                dist = numpy.random.normal(mean, stddev, x)
             else:
-                self._examinees = numpy.random.normal(0, 1, x)
-        elif type(x) == list:
-            self._examinees = numpy.array(x)
-        elif type(x) == numpy.ndarray and x.ndim == 1:
-            self._examinees = x
+                dist = numpy.random.normal(0, 1, x)
+        elif isinstance(x, list):
+            dist = numpy.array(x)
+        elif isinstance(x, numpy.ndarray) and x.ndim == 1:
+            dist = x
         else:
             raise ValueError(
-                'Examinees must be an int, list of floats or one-dimensional numpy array'
+                "Examinees must be an int, list of floats or one-dimensional numpy array"
             )
 
+        return dist
+
     def simulate(
         self,
         initializer: Initializer = None,
         selector: Selector = None,
         estimator: Estimator = None,
         stopper: Stopper = None,
-        verbose: bool = False
+        verbose: bool = False,
     ):
         """Simulates a computerized adaptive testing application to one or more examinees
 
@@ -288,13 +402,13 @@ class Simulator:
 
         >>> from catsim.initialization import RandomInitializer
         >>> from catsim.selection import MaxInfoSelector
-        >>> from catsim.estimation import HillClimbingEstimator
+        >>> from catsim.estimation import NumericalSearchEstimator
         >>> from catsim.stopping import MaxItemStopper
         >>> from catsim.simulation import Simulator
         >>> from catsim.cat import generate_item_bank
         >>> initializer = RandomInitializer()
         >>> selector = MaxInfoSelector()
-        >>> estimator = HillClimbingEstimator()
+        >>> estimator = NumericalSearchEstimator()
         >>> stopper = MaxItemStopper(20)
         >>> Simulator(generate_item_bank(100), 10).simulate(initializer, selector, estimator, stopper)
         """
@@ -308,15 +422,23 @@ class Simulator:
         if stopper is not None:
             self._stopper = stopper
 
+        assert self._initializer is not None
+        assert self._selector is not None
+        assert self._estimator is not None
+        assert self._stopper is not None
+
         for s in [self._initializer, self._selector, self._estimator, self._stopper]:
             s.simulator = self
 
         if verbose:
             print(
                 (
-                    'Starting simulation: {0} {1} {2} {3} {4} items'.format(
-                        self._initializer, self._selector, self._estimator, self._stopper,
-                        self._items.shape[0]
+                    "Starting simulation: {0} {1} {2} {3} {4} items".format(
+                        self._initializer,
+                        self._selector,
+                        self._estimator,
+                        self._stopper,
+                        self._items.shape[0],
                     )
                 )
             )
@@ -342,10 +464,15 @@ class Simulator:
 
                 # simulates the examinee's response via the four-parameter
                 # logistic function
-                response = irt.icc(
-                    true_theta, self.items[selected_item][0], self.items[selected_item][1],
-                    self.items[selected_item][2], self.items[selected_item][3]
-                ) >= numpy.random.uniform()
+                response = (
+                    irt.icc(
+                        true_theta,
+                        self.items[selected_item][0],
+                        self.items[selected_item][1],
+                        self.items[selected_item][2],
+                        self.items[selected_item][3],
+                    ) >= numpy.random.uniform()
+                )
 
                 self._response_vectors[current_examinee].append(response)
 
@@ -369,22 +496,22 @@ class Simulator:
 
                 self._estimations[current_examinee].append(est_theta)
 
-        self._duration = (time.time() - start_time)
+        self._duration = time.time() - start_time
 
         if verbose:
             pbar.close()
-            print('Simulation took {0} seconds'.format(self._duration))
+            print("Simulation took {0} seconds".format(self._duration))
 
         self._bias = cat.bias(self.examinees, self.latest_estimations)
         self._mse = cat.mse(self.examinees, self.latest_estimations)
         self._rmse = cat.rmse(self.examinees, self.latest_estimations)
 
-        # overlap is calculated only if all examinees answered the same amount of items
+        # overlap is computed only if all examinees answered the same amount of items
         # maybe there is a way to calculate it with tests of different lengths,
         # but I did not find it in the literature
         test_size = None
-        len_first = len(self._administered_items[0]) if self._administered_items else None
-        if type(selector) is FiniteSelector:
+        len_first = (len(self._administered_items[0]) if self._administered_items else None)
+        if isinstance(selector, FiniteSelector):
             test_size = selector.test_size
         elif all(len(i) == len_first for i in self._administered_items):
             test_size = len_first
@@ -392,7 +519,7 @@ class Simulator:
             self._overlap_rate = cat.overlap_rate(self.items[:, 4], test_size)
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     import doctest
 
     doctest.testmod()
